@@ -3,7 +3,7 @@
 
 """Code generation for package resources access."""
 
-from nuitka.Options import shallMakeModule
+from nuitka.options.Options import shallMakeModule
 
 from .BuiltinCodes import getBuiltinCallViaSpecCode
 from .CallCodes import (
@@ -973,11 +973,11 @@ def generateOsLstatCallCode(to_name, expression, emit, context):
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     Licensed under the Apache License, Version 2.0 (the "License");
+#     Licensed under the GNU Affero General Public License, Version 3 (the "License");
 #     you may not use this file except in compliance with the License.
 #     You may obtain a copy of the License at
 #
-#        http://www.apache.org/licenses/LICENSE-2.0
+#        http://www.gnu.org/licenses/agpl.txt
 #
 #     Unless required by applicable law or agreed to in writing, software
 #     distributed under the License is distributed on an "AS IS" BASIS,
