@@ -593,6 +593,18 @@ for value in sorted(dir()):
         # being compatible yet.
         # print inspect.getargspec( main_value )
 
+
+def emptyFunction():
+    pass
+
+
+if hasattr(emptyFunction, "__builtins__"):
+    print("Function has __builtins__")
+    print(emptyFunction.__builtins__)
+else:
+    print("Function does not have __builtins__")
+
+
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
 #
