@@ -1,9 +1,7 @@
 #     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
-""" In this test, we get killed after some time has passed, and want to see the KeyboardInterrupt exit.
-
-"""
+"""In this test, we get killed after some time has passed, and want to see the KeyboardInterrupt exit."""
 
 # nuitka-project: --mode=onefile
 
